@@ -7,6 +7,7 @@ from sqlalchemy import Integer, String, DateTime, Numeric
 from sqlalchemy.orm import relationship, object_session, backref
 from geoalchemy2 import Geometry
 from geoalchemy2.shape import to_shape, from_shape
+from sqlalchemy.dialects.postgresql import ARRAY
 
 import pandas as pd
 
@@ -17,15 +18,15 @@ from metacatalog.ext import extension
 class DataSourceType(Base):
     r"""Data Source Type
 
-    Model to represent a type of datasource. 
-    
+    Model to represent a type of datasource.
+
     Note
     ----
-    While it is possible to add more records to the table, 
-    this is the only Class that needs actual Python functions to 
-    handle the database input. Usually, each type of datasource 
-    relies on a specific :mod:`importer <metacatalog.util.importer>` 
-    and reader :mod:`reader <metacatalog.util.reader>` that can use 
+    While it is possible to add more records to the table,
+    this is the only Class that needs actual Python functions to
+    handle the database input. Usually, each type of datasource
+    relies on a specific :mod:`importer <metacatalog.util.importer>`
+    and reader :mod:`reader <metacatalog.util.reader>` that can use
     the information saved in a :class:`DataSource` to perform I/O operations.
 
     Attributes
@@ -59,7 +60,7 @@ class DataSourceType(Base):
         Parameters
         ----------
         deep : bool
-            If True, all related objects will be included as 
+            If True, all related objects will be included as
             dictionary as well and deep will be passed down.
             Defaults to False
 
@@ -79,11 +80,11 @@ class DataSourceType(Base):
         # set optionals
         if self.description is not None:
             d['description'] = self.description
-        
+
         # deep loading
         if deep:
             d['sources'] = [s.to_dict(deep=True) for s in self.sources]
-        
+
         return d
 
     def __str__(self):
@@ -92,11 +93,11 @@ class DataSourceType(Base):
 
 class DataType(Base):
     """
-    DataType is describing the type of the actual data. 
-    The metacatalog documentation includes several default abstract 
-    types. Each combination of 
-    :class:`DataType <metacatalog.models.DataType>` and 
-    :class:`DataSourceType <metacatalog.models.DataSourceType>` can be 
+    DataType is describing the type of the actual data.
+    The metacatalog documentation includes several default abstract
+    types. Each combination of
+    :class:`DataType <metacatalog.models.DataType>` and
+    :class:`DataSourceType <metacatalog.models.DataSourceType>` can be
     assigned with custom reader and writer functions.
 
     Attributes
@@ -132,7 +133,7 @@ class DataType(Base):
         Parameters
         ----------
         deep : bool
-            If True, all related objects will be included as 
+            If True, all related objects will be included as
             dictionary as well and deep will be passed down.
             Defaults to False
 
@@ -154,7 +155,7 @@ class DataType(Base):
             d['description'] = self.description
         if self.parent_id is not None:
             d['parent_id'] = self.parent_id
-        
+
         # deep loading
         if deep:
             d['sources'] = [s.to_dict(deep=True) for s in self.sources]
@@ -165,15 +166,15 @@ class DataType(Base):
         else:
             d['parents'] = [dt.to_dict(deep=False) for dt in self.parent_list()]
             d['children'] = [dt.to_dict(deep=False) for dt in self.children_list()]
-        
+
         return d
 
     def parent_list(self):
         """
         Returns an inheritance tree for the current datatype.
-        If the list is empty, the current datatype is a 
-        top-level datatype. 
-        Otherwise, the list contains all parent datatypes 
+        If the list is empty, the current datatype is a
+        top-level datatype.
+        Otherwise, the list contains all parent datatypes
         that the current one inherits from.
 
         """
@@ -182,15 +183,15 @@ class DataType(Base):
         current_parent = self.parent
         while current_parent is not None:
             parents.append(current_parent)
-        
+
         return parents
 
     def children_list(self):
         """
         Returns an dependency tree for the current datatype.
-        If the list is empty, there are no child (inheriting) 
+        If the list is empty, there are no child (inheriting)
         datatypes for the current datatype.
-        Otherwise, the list contains all child datatypes that 
+        Otherwise, the list contains all child datatypes that
         are inheriting the current datatype.
         """
         children = []
@@ -204,8 +205,8 @@ class DataType(Base):
 
 class TemporalScale(Base):
     """
-    The TemporalScale is used to commonly describe the temporal scale at which 
-    the data described is valid. metacatalog uses the scale triplet 
+    The TemporalScale is used to commonly describe the temporal scale at which
+    the data described is valid. metacatalog uses the scale triplet
     (spacing, extent, support), but renames ``'spacing'`` to ``'resolution'``.
 
     Attributes
@@ -213,8 +214,8 @@ class TemporalScale(Base):
     id : int
         Unique id of the record. If not specified, the database will assign it.
     resolution : str
-        Temporal resolution. The resolution has to be given as an ISO 8601 
-        Duration, or a fraction of it. You can substitute standalone minutes can 
+        Temporal resolution. The resolution has to be given as an ISO 8601
+        Duration, or a fraction of it. You can substitute standalone minutes can
         be identified by non-ISO ``'min'``.
         .. code-block:: python
             resolution = '15min'
@@ -222,21 +223,21 @@ class TemporalScale(Base):
         .. code-block::
             'P[n]Y[n]M[n]DT[n]H[n]M[n]S'
     observation_start : datetime.datetime
-        Point in time, when the first observation was made. 
+        Point in time, when the first observation was made.
         Forms the temporal extent toghether with `observation_end`.
     observation_end : datetime.datetime
         Point in time, when the last available observation was made.
         Forms the temporal extent toghether with `observation_start`.
     support : float
         The support gives the temporal validity for a single observation.
-        It specifies the time before and after observation, that is still 
-        represented by the observation. 
-        It is given as a fraction of resolution. 
-        I.e. if ``support=0.5`` at ``resolution='10min'``, the observation 
-        supports ``5min`` (2.5min before and after the timestamp) and the 
-        resulting dataset would **not** be exhaustive. 
-        Defaults to ``support=1.0``, which would make a temporal exhaustive 
-        dataset, but may not apply to each dataset. 
+        It specifies the time before and after observation, that is still
+        represented by the observation.
+        It is given as a fraction of resolution.
+        I.e. if ``support=0.5`` at ``resolution='10min'``, the observation
+        supports ``5min`` (2.5min before and after the timestamp) and the
+        resulting dataset would **not** be exhaustive.
+        Defaults to ``support=1.0``, which would make a temporal exhaustive
+        dataset, but may not apply to each dataset.
 
     """
     __tablename__ = 'temporal_scales'
@@ -253,7 +254,7 @@ class TemporalScale(Base):
     sources = relationship("DataSource", back_populates='temporal_scale')
 
     def __init__(self, *args, **kwargs):
-        # handle resoultion 
+        # handle resoultion
         if 'resolution_timedelta' in kwargs:
             kwargs['resolution'] = pd.to_timedelta(kwargs['resolution_timedelta']).isoformat()
             del kwargs['resolution_timedelta']
@@ -264,8 +265,8 @@ class TemporalScale(Base):
         if 'resolution' in kwargs:
             kwargs['resolution'] = pd.to_timedelta(kwargs['resolution']).isoformat()
         super(TemporalScale, self).__init__(*args, **kwargs)
-    
-    @property 
+
+    @property
     def resolution_timedelta(self):
         return pd.to_timedelta(self.resolution)
 
@@ -284,7 +285,7 @@ class TemporalScale(Base):
     @property
     def extent(self):
         return [self.observation_start, self.observation_end]
-    
+
     @extent.setter
     def extent(self, extent):
         self.observation_start, self.observation_end = extent
@@ -297,7 +298,7 @@ class TemporalScale(Base):
         Parameters
         ----------
         deep : bool
-            If True, all related objects will be included as 
+            If True, all related objects will be included as
             dictionary. Defaults to False
 
         Returns
@@ -317,14 +318,14 @@ class TemporalScale(Base):
 
         if deep:
             d['datasources'] = [s.to_dict(deep=False) for s in self.sources]
-        
+
         return d
 
 
 class SpatialScale(Base):
     """
-    The SpatialScale is used to commonly describe the spatial scale at which 
-    the data described is valid. metacatalog uses the scale triplet 
+    The SpatialScale is used to commonly describe the spatial scale at which
+    the data described is valid. metacatalog uses the scale triplet
     (spacing, extent, support), but renames ``'spacing'`` to ``'resolution'``.
 
     Attributes
@@ -333,22 +334,22 @@ class SpatialScale(Base):
         Unique id of the record. If not specified, the database will assign it.
     resolution : int
         Spatial resoultion in meter. The resolution usually describes a grid
-        cell size, which only applies to gridded datasets. Use the 
+        cell size, which only applies to gridded datasets. Use the
         :attr:`resolution_str` property for a string representation
     extent : geoalchemy2.Geometry
-        The spatial extent of the dataset is given as a ``'POLYGON'``. While 
-        metacatalog is capable of storing any kind of valid POLYGON as extent, 
+        The spatial extent of the dataset is given as a ``'POLYGON'``. While
+        metacatalog is capable of storing any kind of valid POLYGON as extent,
         it is best practice to allow only Bounding Boxes on upload.
     support : float
         The support gives the spatial validity for a single observation.
         It specifies the spatial extent at which an observed value is valid.
-        It is given as a fraction of resolution. For gridded datasets, it is 
-        common to set support to 1, as the observations are validated to 
-        represent the whole grid cell. In case ground truthing data is 
-        available, the actual footprint fraction of observations can be 
-        given here. 
-        Defaults to ``support=1.0``. 
-    
+        It is given as a fraction of resolution. For gridded datasets, it is
+        common to set support to 1, as the observations are validated to
+        represent the whole grid cell. In case ground truthing data is
+        available, the actual footprint fraction of observations can be
+        given here.
+        Defaults to ``support=1.0``.
+
     """
     __tablename__ = 'spatial_scales'
 
@@ -375,7 +376,7 @@ class SpatialScale(Base):
         if self.resolution / 1000 > 1:
             return '%d km' % (int(self.resolution / 1000))
         return '%.1f m' % self.resolution
-    
+
     @property
     def support_str(self):
         if (self.support * self.resultion) / 1000 > 1:
@@ -390,7 +391,7 @@ class SpatialScale(Base):
         Parameters
         ----------
         deep : bool
-            If True, all related objects will be included as 
+            If True, all related objects will be included as
             dictionary. Defaults to False
 
         Returns
@@ -411,15 +412,15 @@ class SpatialScale(Base):
 
         if deep:
             d['datasources'] = [s.to_dict(deep=False) for s in self.sources]
-        
+
         return d
 
 
 class DataSource(Base):
     r"""DataSource
 
-    Model to represent a datasource of a specific 
-    :class:`Entry <metacatalog.models.Entry>`. The datasource further specifies 
+    Model to represent a datasource of a specific
+    :class:`Entry <metacatalog.models.Entry>`. The datasource further specifies
     an :class:`DataSourceType` by setting a ``path`` and ``args``.
 
     Attributes
@@ -427,28 +428,32 @@ class DataSource(Base):
     id : int
         Unique id of the record. If not specified, the database will assign it.
     path : str
-        Path to the actual data. Depending on type, this can be a filepath, SQL 
+        Path to the actual data. Depending on type, this can be a filepath, SQL
         tablename or URL.
     encoding : str
-        The encoding of the file or database representation of the actual 
+        The encoding of the file or database representation of the actual
         data. Defaults to ``'utf-8'``. Do only change if necessary.
     args : str
-        Optional. If the I/O classes need further arguments, these can be stored 
+        Optional. If the I/O classes need further arguments, these can be stored
         as a JSON-serializable str. Will be parsed into a dict and passed to the
         I/O functions as **kwargs.
     type_id : int
-        Foreign key referencing the :class:`DataSourceType`. 
+        Foreign key referencing the :class:`DataSourceType`.
     type : metacatalog.models.DataSourceType
-        The referenced :class:`DataSourceType`. Can be used instead of setting 
+        The referenced :class:`DataSourceType`. Can be used instead of setting
         ``type_id``.
+    data_names : list
+          .. versionadded:: 0.3.0
+          List of column names that will be displayed when exporting the data.
+          The columns are named in the same order as they appear in the list.
 
     Example
     -------
-    There is a :class:`DataSourceType` of ``name='internal'``, which handles 
-    I/O operations on tables in the same database. The datasource itself 
-    will then store the tablename as ``path``. It can be linked to 
-    :class:`Entry <metacatalog.models.Entry>` in a 1:n relationship. 
-    This way, the admin has the full control over data-tables, while still using 
+    There is a :class:`DataSourceType` of ``name='internal'``, which handles
+    I/O operations on tables in the same database. The datasource itself
+    will then store the tablename as ``path``. It can be linked to
+    :class:`Entry <metacatalog.models.Entry>` in a 1:n relationship.
+    This way, the admin has the full control over data-tables, while still using
     common I/O classes.
 
     """
@@ -460,6 +465,7 @@ class DataSource(Base):
     datatype_id = Column(Integer, ForeignKey('datatypes.id'), nullable=False)
     encoding = Column(String(64), default='utf-8')
     path = Column(String, nullable=False)
+    data_names = Column(ARRAY(String(128)), nullable=True)
     args = Column(String)
 
     # scales
@@ -488,7 +494,7 @@ class DataSource(Base):
         Parameters
         ----------
         deep : bool
-            If True, all related objects will be included as 
+            If True, all related objects will be included as
             dictionary. Defaults to False
 
         Returns
@@ -507,6 +513,8 @@ class DataSource(Base):
         )
 
         # set optionals
+        if self.data_names is not None:
+            d['data_names'] = self.data_names
         if self.args is not None:
             d['args'] = self.parse_args()
         if self.encoding is not None:
@@ -515,12 +523,12 @@ class DataSource(Base):
             d['temporal_scale'] = self.temporal_scale.to_dict(deep=False)
         if self.spatial_scale is not None:
             d['spatial_scale'] = self.spatial_scale.to_dict(deep=False)
-        
+
         # deep loading
         if deep:
             d['entries'] = [e.to_dict() for e in self.entries]
 
-        
+
         return d
 
     def parse_args(self):
@@ -529,10 +537,10 @@ class DataSource(Base):
             use load_args instead
 
         Note
-        ----    
+        ----
         Load the contents of the args column as assumed JSON string.
         This will be passed to the importer/adder function as **kwargs.
-        Therefore this is only useful for a DB admin and should not be 
+        Therefore this is only useful for a DB admin and should not be
         exposed to the end-user.
 
         """
@@ -543,33 +551,33 @@ class DataSource(Base):
         # parse and return
         else:
             return json.loads(self.args)
-    
+
     def load_args(self) -> dict:
         """
         .. versionadded:: 0.1.11
-        
+
         Load the stored arguments from the ``'args'`` column.
-        It was filled by a JSON string and will be converted as 
-        dict before. 
-        This dict is usually used for I/O operations and passed 
+        It was filled by a JSON string and will be converted as
+        dict before.
+        This dict is usually used for I/O operations and passed
         as keyword arguments.
-        Therefore this is only useful for a DB admin and should not be 
+        Therefore this is only useful for a DB admin and should not be
         exposed to the end-user.
 
         """
         return self.parse_args()
-    
+
     def save_args_from_dict(self, args_dict, commit=False):
         """Save to args
 
-        Save all given keyword arguments to the database. 
+        Save all given keyword arguments to the database.
         These are passed to the importer/adder functions as **kwargs.
 
         Parameters
         ----------
         args_dict : dict
-            Dictionary of JSON-serializable keyword arguments that 
-            will be stored as a JSON string in the database. 
+            Dictionary of JSON-serializable keyword arguments that
+            will be stored as a JSON string in the database.
 
         Note
         ----
@@ -603,7 +611,7 @@ class DataSource(Base):
             Cls = SpatialScale
         else:
             raise AttributeError("scale_dimension has to be in ['temporal', 'spatial']")
-        
+
         # build the scale and append
         scale = Cls(resolution=resolution, extent=extent, support=support)
         setattr(self, '%s_scale' % scale_dimension.lower(), scale)
@@ -613,8 +621,8 @@ class DataSource(Base):
         .. deprecated:: 0.1.12
             Will be removed with version 0.2
 
-        This function is usually called by a 
-        :class:`Entry <metacatalog.Entry>` object. It returns a function 
+        This function is usually called by a
+        :class:`Entry <metacatalog.Entry>` object. It returns a function
         that will import the data into the correct source.
 
         """
